const each = require('jest-each').default;

const config = require('../mocks/json/config');
const CookieUtils = require('../CookieUtils');

describe('CookieUtils', () => {
  describe('generateUrlRedirect', () => {
    each([
      [
        {
          url: 'https://google.com',
        },
        'https://cookies.asksuite.com/redirect?ck__ASKSUITE__LastUrl=https%3A%2F%2Fgoogle.com&url=https%3A%2F%2Fgoogle.com',
      ],
      [
        {
          url: 'https://google.com',
          quotationIdentifier: '79b1787d-f28e-443a-8ddc-ccb43a2fe7e9',
        },
        'https://cookies.asksuite.com/redirect?ck__ASKSUITE__QuotationIdentifier=79b1787d-f28e-443a-8ddc-ccb43a2fe7e9&ck__ASKSUITE__LastUrl=https%3A%2F%2Fgoogle.com&url=https%3A%2F%2Fgoogle.com',
      ],
      [
        {
          url:
            'https://myreservations.omnibees.com/default.aspx?q=4937&diff=false&CheckIn=21072020&CheckOut=23072020&Code=&group_code=&loyality_card=&NRooms=1&ad=2&ch=0&ag=-&lang=pt-br&utm_source=asksuite',
          quotationIdentifier: '79b1787d-f28e-443a-8ddc-ccb43a2fe7e9',
        },
        'https://cookies.asksuite.com/redirect?ck__ASKSUITE__QuotationIdentifier=79b1787d-f28e-443a-8ddc-ccb43a2fe7e9&ck__ASKSUITE__LastUrl=https%3A%2F%2Fmyreservations.omnibees.com%2Fdefault.aspx%3Fq%3D4937%26diff%3Dfalse%26CheckIn%3D21072020%26CheckOut%3D23072020%26Code%3D%26group_code%3D%26loyality_card%3D%26NRooms%3D1%26ad%3D2%26ch%3D0%26ag%3D-%26lang%3Dpt-br%26utm_source%3Dasksuite&url=https%3A%2F%2Fmyreservations.omnibees.com%2Fdefault.aspx%3Fq%3D4937%26diff%3Dfalse%26CheckIn%3D21072020%26CheckOut%3D23072020%26Code%3D%26group_code%3D%26loyality_card%3D%26NRooms%3D1%26ad%3D2%26ch%3D0%26ag%3D-%26lang%3Dpt-br%26utm_source%3Dasksuite',
      ],
    ]).it("when the input is '%s'", (input, expected) => {
      const result = CookieUtils.generateUrlRedirect(input, config);
      expect(result).toEqual(expected);
    });
  });
});
