const AWS = require('aws-sdk');

class AWSLambdaCaller {
  constructor(awsConfig, otherParams = {}) {
    AWS.config.update(awsConfig);
    this.otherParams = otherParams || {};
  }

  async call(jsonObj, functionName) {
    const lambda = new AWS.Lambda(this.otherParams);

    const params = {
      FunctionName: functionName,
      Payload: JSON.stringify(jsonObj),
    };
    try {
      return await lambda
        .invoke(params)
        .promise()
        .then(value => {
          return JSON.parse(value.Payload);
        });
    } catch (e) {
      console.warn('Error, cannot invoke lambda ' + functionName, e);
      throw e;
    }
  }
}

module.exports = AWSLambdaCaller;
