const Promise = require('bluebird');
const _ = require('lodash');

const shouldIgnorePath = (fullPath, ignorePaths) => {
  return _.some(ignorePaths, path => {
    return pathToString(fullPath).endsWith(pathToString(path));
  });
};

const pathToString = path => _.join(path, '.');

const asyncMapDeep = (array, key, mapFn, ignoreBranches, ignorePaths, path = []) =>
  Promise.map(
    array,
    value =>
      _.isObject(value) || _.isArray(value)
        ? asyncMapValuesDeep(value, mapFn, ignoreBranches, ignorePaths, path)
        : mapFn(value, key),
    { concurrency: 100 },
  );

const asyncMapValuesDeep = (object, mapFn, ignoreBranches, ignorePaths, path = []) =>
  Promise.props(
    _.mapValues(object, (value, key) => {
      const currentPath = _.clone(path);
      currentPath.push(key);
      if (_.includes(ignoreBranches, key) || shouldIgnorePath(currentPath, ignorePaths)) {
        return value;
      }
      return _.isArray(value)
        ? asyncMapDeep(value, key, mapFn, ignoreBranches, ignorePaths, currentPath)
        : _.isObject(value)
        ? asyncMapValuesDeep(value, mapFn, ignoreBranches, ignorePaths, currentPath)
        : mapFn(value, key);
    }),
  );

const removePreventTranslationMarkers = text => {
  if (!text || !_.isString(text)) {
    return text;
  }

  const markerRegex = /\[\[[^\[\]]*\]\]/g;
  const markers = text.match(markerRegex);

  if (markers && markers.length) {
    markers.forEach(marker => {
      text = text.replace(marker, marker.replace('[[', '').replace(']]', ''));
    });
  }

  return text;
};

module.exports = {
  asyncMapValuesDeep,
  removePreventTranslationMarkers,
};
